# -*- coding: utf-8 -*-
import colorlover as cl
import dash
import dash_core_components as dcc
import dash_html_components as html
import plotly.graph_objs as go

import sys, time, json, random, logging, codecs
from datetime import datetime


# open the file in utf-8 encoding 
with codecs.open("election2016_20160926_top30_hashtags_hourly_trends.json", "r", encoding="utf-8") as fp:
    data = json.load(fp) # load the content into a dictionary called `data`

# extract top N hashtags (up to 30)    
top = 6
top_hashtags = data["top_hashtags"][:top]

# define colors 
colors = cl.scales[str(top%11)]['qual']['Paired']
hashtag_colors = {}
for idx, hashtag in enumerate(top_hashtags):
    hashtag_colors[hashtag] = colors[idx % 11]

# current highlighting trends (default to be empty, which will show all trends)
highlighted_hashtags = set()

# initialize Dash app
app = dash.Dash()

# set layout of the page
app.layout = html.Div(children=[

    # set the page heading
    html.H1(children='Trigger changes of one plot from another plot'),

    # set the description underneath the heading
    html.Div(children='''
        A demo to show how to make the plots connected.
    '''),
   
    # put charts in a div 
    html.Div(children=[
        
        # the first graph is a barchart
        dcc.Graph(
            id='top-hashtag-barchart',
            figure={
                # configure the data
                'data': [{
                        # set x to be hashtags, and y to be the counts. We use bars to represent our data
                        'x': top_hashtags, 
                        'y': [sum(data["trends"][hashtag]) for hashtag in top_hashtags], 
                        'type': 'bar',
                        'marker': {
                            'color': colors
                        }
                    },                    
                ],
                # configure the layout of the visualization -- set the title to be Election2016 hashtag Trends
                'layout': {
                    'title': 'Election2016 top %s hashtags on Sept 26, 2016' % top
                },
            },
            clickData={"points": []},
            config={'displayModeBar': False}, 
            style={
               'width': 500
            }
        ),
        
        
        # the second graph is a time series 
        dcc.Graph(
            id='trend-series',
            config={'displayModeBar': False},
            clickData={"points": []}, style={
               'width': 800
            }
        ), 
    ], style={
        'display': 'flex',
        'height': 500
    })
    
], style={
   'text-align': 'center'
})

# define interaction: when click on the bar chart, the highlighting of the time series will be changed
@app.callback(
    dash.dependencies.Output('trend-series', 'figure'),
    [dash.dependencies.Input('top-hashtag-barchart', 'clickData')])
def update_graph(click_data):
    print click_data
    series = []
   
    for point in click_data["points"]:
        if point["x"] not in highlighted_hashtags:            
            highlighted_hashtags.add(point["x"])
        elif point["x"] in highlighted_hashtags:
            highlighted_hashtags.remove(point["x"])
      
    for idx, hashtag in enumerate(top_hashtags):
        if len(highlighted_hashtags) == 0 or hashtag in highlighted_hashtags:
            series.append(
                go.Scatter(
                    x=[datetime(2016, 9, 26, h) for h in range(24)],
                    y=data["trends"][hashtag],
                    mode='lines+markers',
                    name=hashtag,
                    marker= {
                        'color': hashtag_colors[hashtag]
                    }                
                ) 
            )        
            
    return {
            'data': series,
            'layout': {
                'title': 'Election2016 hashtag Trends'
            }
        }    

    
if __name__ == '__main__':    
    app.run_server(debug=True)